@@ -12,24 +12,25 @@ import (
 	"time"
 
 	"github.com/stretchr/testify/require"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/config"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage/keyvalue"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage/keyvalue/databasemgr"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage/storagemgr/partition/log"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage/wal"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/helper"
-	logrus "gitlab.com/gitlab-org/gitaly/v16/internal/log"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/testhelper"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/testhelper/testcfg"
-	"gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/config"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage/keyvalue"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage/keyvalue/databasemgr"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage/storagemgr/partition/log"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage/wal"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/helper"
+	logrus "gitlab.com/gitlab-org/gitaly/v18/internal/log"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/testhelper"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/testhelper/testcfg"
+	"gitlab.com/gitlab-org/gitaly/v18/proto/go/gitalypb"
+	"go.etcd.io/raft/v3"
 	"go.etcd.io/raft/v3/raftpb"
 	"golang.org/x/exp/slices"
 	"google.golang.org/grpc"
 )
 
 const (
-	waitTimeout = 5 * time.Second
+	waitTimeout = 10 * time.Second
 )
 
 func raftConfigsForTest(t *testing.T) config.Raft {
@@ -231,15 +232,16 @@ func TestReplica_Initialize(t *testing.T) {
 		require.NoError(t, localLog.Close())
 
 		// Now create a Raft log store pointing to the same directories
-		logStore, err := NewReplicaLogStore(storageName, partitionID, raftCfg, db, stagingDir, stateDir, &mockConsumer{}, posTracker, logger, NewMetrics())
+		logStore, err := NewReplicaLogStore(storageName, partitionID, raftCfg, db, stagingDir, stateDir, &MockConsumer{}, posTracker, logger, NewMetrics())
 		require.NoError(t, err)
 
 		raftNode, err := NewNode(cfg, logger, dbMgr, nil)
 		require.NoError(t, err)
 
 		raftFactory := DefaultFactoryWithNode(raftCfg, raftNode, WithEntryRecorder(recorder))
+		ctx = storage.ContextWithPartitionInfo(ctx, NewPartitionKey(storageName, partitionID), 1, "")
 
-		mgr, err := raftFactory(1, storageName, NewPartitionKey(storageName, partitionID), logStore, logger, metrics)
+		mgr, err := raftFactory(ctx, storageName, logStore, logger, metrics)
 		require.NoError(t, err)
 		defer func() { require.NoError(t, mgr.Close()) }()
 
@@ -309,7 +311,7 @@ func TestReplica_Initialize(t *testing.T) {
 		require.NoError(t, localLog.Close())
 
 		// Now create a Raft log store pointing to the same directories
-		logStore, err := NewReplicaLogStore(storageName, partitionID, raftCfg, db, stagingDir, stateDir, &mockConsumer{}, posTracker, logger, NewMetrics())
+		logStore, err := NewReplicaLogStore(storageName, partitionID, raftCfg, db, stagingDir, stateDir, &MockConsumer{}, posTracker, logger, NewMetrics())
 		require.NoError(t, err)
 
 		raftNode, err := NewNode(cfg, logger, dbMgr, nil)
@@ -317,7 +319,8 @@ func TestReplica_Initialize(t *testing.T) {
 
 		raftFactory := DefaultFactoryWithNode(raftCfg, raftNode, WithEntryRecorder(recorder))
 
-		mgr, err := raftFactory(1, storageName, NewPartitionKey(storageName, partitionID), logStore, logger, metrics)
+		ctx = storage.ContextWithPartitionInfo(ctx, NewPartitionKey(storageName, partitionID), 1, "")
+		mgr, err := raftFactory(ctx, storageName, logStore, logger, metrics)
 		require.NoError(t, err)
 		defer func() { require.NoError(t, mgr.Close()) }()
 
@@ -354,7 +357,7 @@ func TestReplica_Initialize(t *testing.T) {
 		metrics := NewMetrics()
 
 		// PHASE 1: Create a new partition with Raft enabled
-		logStore1, err := NewReplicaLogStore(storageName, partitionID, raftCfg, db, stagingDir, stateDir, &mockConsumer{}, log.NewPositionTracker(), logger, NewMetrics())
+		logStore1, err := NewReplicaLogStore(storageName, partitionID, raftCfg, db, stagingDir, stateDir, &MockConsumer{}, log.NewPositionTracker(), logger, NewMetrics())
 		require.NoError(t, err)
 
 		raftNode, err := NewNode(cfg, logger, dbMgr, nil)
@@ -362,7 +365,8 @@ func TestReplica_Initialize(t *testing.T) {
 
 		raftFactory := DefaultFactoryWithNode(raftCfg, raftNode, WithEntryRecorder(recorder))
 
-		mgr1, err := raftFactory(1, storageName, NewPartitionKey(storageName, partitionID), logStore1, logger, metrics)
+		ctx = storage.ContextWithPartitionInfo(ctx, NewPartitionKey(storageName, partitionID), 1, "")
+		mgr1, err := raftFactory(ctx, storageName, logStore1, logger, metrics)
 		require.NoError(t, err)
 
 		// Initialize the raft replica
@@ -421,10 +425,11 @@ func TestReplica_Initialize(t *testing.T) {
 		}
 
 		// PHASE 3: Re-enable Raft
-		logStore2, err := NewReplicaLogStore(storageName, partitionID, raftCfg, db, stagingDir, stateDir, &mockConsumer{}, log.NewPositionTracker(), logger, NewMetrics())
+		logStore2, err := NewReplicaLogStore(storageName, partitionID, raftCfg, db, stagingDir, stateDir, &MockConsumer{}, log.NewPositionTracker(), logger, NewMetrics())
 		require.NoError(t, err)
 
-		mgr2, err := raftFactory(1, storageName, NewPartitionKey(storageName, partitionID), logStore2, logger, metrics)
+		ctx = storage.ContextWithPartitionInfo(ctx, NewPartitionKey(storageName, partitionID), 1, "")
+		mgr2, err := raftFactory(ctx, storageName, logStore2, logger, metrics)
 		require.NoError(t, err)
 
 		// Re-initialize Raft with the highest LSN
@@ -699,7 +704,7 @@ func TestReplica_AppendLogEntry(t *testing.T) {
 
 		db, dbMgr := dbSetup(t, ctx, cfg, testhelper.TempDir(t), storageName, logger)
 		// Create a Raft log store
-		logStore, err := NewReplicaLogStore(storageName, partitionID, raftCfg, db, stagingDir, stateDir, &mockConsumer{}, posTracker, logger, metrics)
+		logStore, err := NewReplicaLogStore(storageName, partitionID, raftCfg, db, stagingDir, stateDir, &MockConsumer{}, posTracker, logger, metrics)
 		require.NoError(t, err)
 
 		raftNode, err := NewNode(cfg, logger, dbMgr, nil)
@@ -708,10 +713,10 @@ func TestReplica_AppendLogEntry(t *testing.T) {
 		raftFactory := DefaultFactoryWithNode(raftCfg, raftNode, WithEntryRecorder(recorder), WithOpTimeout(1*time.Nanosecond))
 
 		// Create replica with very short operation timeout
+		ctx = storage.ContextWithPartitionInfo(ctx, NewPartitionKey(storageName, partitionID), 1, "")
 		mgr, err := raftFactory(
-			1,
+			ctx,
 			storageName,
-			NewPartitionKey(storageName, partitionID),
 			logStore,
 			logger,
 			metrics,
@@ -820,7 +825,7 @@ func TestReplica_AppendLogEntry_CrashRecovery(t *testing.T) {
 		db, err := dbMgr.GetDB(cfg.Storages[0].Name)
 		require.NoError(t, err)
 
-		logStore, err := NewReplicaLogStore(storageName, partitionID, raftCfg, db, stagingDir, stateDir, &mockConsumer{}, log.NewPositionTracker(), logger, metrics)
+		logStore, err := NewReplicaLogStore(storageName, partitionID, raftCfg, db, stagingDir, stateDir, &MockConsumer{}, log.NewPositionTracker(), logger, metrics)
 		require.NoError(t, err)
 
 		raftNode, err := NewNode(cfg, logger, dbMgr, nil)
@@ -829,7 +834,8 @@ func TestReplica_AppendLogEntry_CrashRecovery(t *testing.T) {
 		raftFactory := DefaultFactoryWithNode(raftCfg, raftNode, WithEntryRecorder(recorder))
 
 		// Configure replica
-		mgr, err := raftFactory(1, storageName, NewPartitionKey(storageName, partitionID), logStore, logger, metrics)
+		ctx = storage.ContextWithPartitionInfo(ctx, NewPartitionKey(storageName, partitionID), 1, "")
+		mgr, err := raftFactory(ctx, storageName, logStore, logger, metrics)
 		require.NoError(t, err)
 
 		for _, f := range setupFuncs {
@@ -874,7 +880,7 @@ func TestReplica_AppendLogEntry_CrashRecovery(t *testing.T) {
 		db, err := dbMgr.GetDB(env.cfg.Storages[0].Name)
 		require.NoError(t, err)
 
-		logStore, err := NewReplicaLogStore(env.storageName, env.partitionID, raftCfg, db, env.stagingDir, env.stateDir, &mockConsumer{}, log.NewPositionTracker(), logger, env.metrics)
+		logStore, err := NewReplicaLogStore(env.storageName, env.partitionID, raftCfg, db, env.stagingDir, env.stateDir, &MockConsumer{}, log.NewPositionTracker(), logger, env.metrics)
 		require.NoError(t, err)
 
 		raftNode, err := NewNode(env.cfg, logger, dbMgr, nil)
@@ -882,7 +888,8 @@ func TestReplica_AppendLogEntry_CrashRecovery(t *testing.T) {
 
 		raftFactory := DefaultFactoryWithNode(raftCfg, raftNode, WithEntryRecorder(env.recorder))
 
-		recoveryMgr, err := raftFactory(1, env.storageName, NewPartitionKey(env.storageName, env.partitionID), logStore, logger, env.metrics)
+		ctx = storage.ContextWithPartitionInfo(ctx, NewPartitionKey(env.storageName, env.partitionID), 1, "")
+		recoveryMgr, err := raftFactory(ctx, env.storageName, logStore, logger, env.metrics)
 		require.NoError(t, err)
 
 		// Initialize with the last known LSN
@@ -1663,7 +1670,8 @@ func TestReplica_StorageConnection(t *testing.T) {
 	// Create factory that connects replicas to storage
 	raftFactory := DefaultFactoryWithNode(cfg.Raft, raftNode)
 
-	replica, err := raftFactory(1, storageName, NewPartitionKey(storageName, partitionID), logStore, logger, NewMetrics())
+	ctx = storage.ContextWithPartitionInfo(ctx, NewPartitionKey(storageName, partitionID), 1, "")
+	replica, err := raftFactory(ctx, storageName, logStore, logger, NewMetrics())
 	require.NoError(t, err)
 	t.Cleanup(func() { require.NoError(t, replica.Close()) })
 
@@ -1694,14 +1702,16 @@ func TestReplica_StorageConnection(t *testing.T) {
 	})
 
 	t.Run("multiple replicas for same partition key", func(t *testing.T) {
-		duplicateReplica, err := raftFactory(1, storageName, NewPartitionKey(storageName, partitionID), logStore, logger, NewMetrics())
+		ctx = storage.ContextWithPartitionInfo(ctx, NewPartitionKey(storageName, partitionID), 1, "")
+		duplicateReplica, err := raftFactory(ctx, storageName, logStore, logger, NewMetrics())
 		require.NoError(t, err)
 		require.NotNil(t, duplicateReplica)
 	})
 
 	t.Run("Register different replicas for different partition keys", func(t *testing.T) {
 		partitionID := storage.PartitionID(2)
-		replicaTwo, err := raftFactory(1, storageName, NewPartitionKey(storageName, partitionID), logStore, logger, NewMetrics())
+		ctx = storage.ContextWithPartitionInfo(ctx, NewPartitionKey(storageName, partitionID), 1, "")
+		replicaTwo, err := raftFactory(ctx, storageName, logStore, logger, NewMetrics())
 		require.NoError(t, err)
 		require.NotNil(t, replicaTwo)
 	})
@@ -1741,11 +1751,11 @@ func TestReplica_AddNode(t *testing.T) {
 		}, timeout, 5*time.Millisecond, "replica should become leader")
 	}
 
-	// waitForVoters waits for both replicas to have the expected number of voters
-	waitForReplicaLeader := func(t *testing.T, replicaOne, replicaTwo *Replica, expectedVoters int, timeout time.Duration) {
+	// waitForReplicaLeader waits for both replicas to have the same leader
+	waitForReplicaLeader := func(t *testing.T, replicaOne, replicaTwo *Replica, timeout time.Duration) {
 		require.Eventually(t, func() bool {
 			return replicaTwo.leadership.GetLeaderID() == replicaOne.memberID
-		}, timeout, 5*time.Millisecond, "configuration should stabilize with %d voters", expectedVoters)
+		}, timeout, 5*time.Millisecond, "replica two should have the same leader as replica one")
 	}
 
 	drainNotificationQueues := func(t *testing.T, replicas ...*Replica) {
@@ -1817,8 +1827,9 @@ func TestReplica_AddNode(t *testing.T) {
 		})
 
 		require.NoError(t, err)
+		destination := "default"
 
-		err = replica.AddNode(ctx, replicaTwoAddress)
+		err = replica.AddNode(ctx, replicaTwoAddress, destination)
 		require.NoError(t, err, "adding node should succeed when leader")
 
 		require.Eventually(t, func() bool {
@@ -1833,7 +1844,7 @@ func TestReplica_AddNode(t *testing.T) {
 
 		drainNotificationQueues(t, replica, replicaTwo)
 
-		waitForReplicaLeader(t, replica, replicaTwo, 2, waitTimeout)
+		waitForReplicaLeader(t, replica, replicaTwo, waitTimeout)
 
 		testhelper.RequirePromMetrics(t, metrics, `
 			# HELP gitaly_raft_log_entries_processed Rate of log entries processed.
@@ -1859,12 +1870,14 @@ func TestReplica_AddNode(t *testing.T) {
 		metrics := NewMetrics()
 		partitionID := storage.PartitionID(1)
 
-		replica, _, srv := createTestNode(t, ctx, 1, partitionID, raftCfg, metrics)
+		replica, socketPath, srv := createTestNode(t, ctx, 1, partitionID, raftCfg, metrics)
 		defer func() {
 			srv.Stop()
 			require.NoError(t, replica.Close())
 		}()
 
+		replicaOneAddress := "unix://" + socketPath
+
 		waitForLeadership(t, replica, waitTimeout)
 
 		raftEnabledStorage := replica.raftEnabledStorage
@@ -1890,18 +1903,44 @@ func TestReplica_AddNode(t *testing.T) {
 		lastMemberID := uint64(3)
 
 		for i := uint64(2); i <= lastMemberID; i++ {
+			// create multiple replicas with new addresses
 			replica, socketPath, srv := createTestNode(t, ctx, i, partitionID, raftCfg, metrics)
 
 			servers = append(servers, srv)
 			address := "unix://" + socketPath
 			destinationAddresses = append(destinationAddresses, address)
 			addressesToReplicas[address] = replica
+
+			// Register the replica so it can receive Raft messages from other nodes.
+			// Without registration, the transport layer cannot route messages to this replica,
+			// preventing it from learning about the current leader.
+			replicaRegistry := replica.raftEnabledStorage.GetReplicaRegistry()
+			replicaRegistry.RegisterReplica(partitionKey, replica)
+
+			replicaRoutingTable := replica.raftEnabledStorage.GetRoutingTable()
+			err := replicaRoutingTable.UpsertEntry(RoutingTableEntry{
+				Replicas: []*gitalypb.ReplicaID{
+					{
+						PartitionKey: partitionKey,
+						Type:         gitalypb.ReplicaID_REPLICA_TYPE_VOTER,
+						MemberId:     1,
+						Metadata: &gitalypb.ReplicaID_Metadata{
+							Address: replicaOneAddress,
+						},
+					},
+				},
+				Term:  1,
+				Index: 2,
+			})
+			require.NoError(t, err)
 		}
 
+		destination := "default"
+
 		// Propose concurrent membership changes. Same member ID but different address.
 		var errs []error
 		for i := 0; i < 2; i++ {
-			err := replica.proposeMembershipChange(ctx, string(addVoter), lastMemberID, ConfChangeAddNode, &gitalypb.ReplicaID_Metadata{
+			err := replica.proposeMembershipChange(ctx, string(addVoter), destination, lastMemberID, ConfChangeAddNode, &gitalypb.ReplicaID_Metadata{
 				Address: destinationAddresses[i],
 			})
 			errs = append(errs, err)
@@ -1917,7 +1956,7 @@ func TestReplica_AddNode(t *testing.T) {
 			require.ErrorContains(t, err, "configuration change timed out after")
 		}
 
-		require.Equal(t, successfulProposals, 1, "only one of the proposals should succeed")
+		require.Equal(t, 1, successfulProposals, "only one of the proposals should succeed")
 
 		var addedAddress string
 		require.Eventually(t, func() bool {
@@ -1946,7 +1985,7 @@ func TestReplica_AddNode(t *testing.T) {
 		addedReplica := addressesToReplicas[addedAddress]
 
 		// Wait for voters count to stabilize between both replicas
-		waitForReplicaLeader(t, replica, addedReplica, 2, waitTimeout)
+		waitForReplicaLeader(t, replica, addedReplica, waitTimeout)
 
 		drainNotificationQueues(t, replica, addedReplica)
 
@@ -1988,8 +2027,8 @@ func TestReplica_AddNode(t *testing.T) {
 		// Wait for the replica to elect itself as leader
 		waitForLeadership(t, replica, waitTimeout)
 
-		err := replica.RemoveNode(ctx, 999)
-		require.EqualError(t, err, "checking member ID: translating member ID: no address found for memberID 999")
+		err := replica.RemoveNode(ctx, 999, storageName)
+		require.EqualError(t, err, "translating member ID: no address found for memberID 999")
 	})
 
 	t.Run("fails when node is not leader", func(t *testing.T) {
@@ -2010,7 +2049,7 @@ func TestReplica_AddNode(t *testing.T) {
 		// Set a random leader ID to simulate a non-leader
 		replica.leadership.SetLeader(999, false)
 
-		err := replica.AddNode(ctx, "gitaly-node-2:8075")
+		err := replica.AddNode(ctx, "gitaly-node-2:8075", storageName)
 		require.EqualError(t, err, "replica is not the leader", "adding node should fail when not leader")
 
 		testhelper.RequirePromMetrics(t, metrics, `
@@ -2020,3 +2059,208 @@ func TestReplica_AddNode(t *testing.T) {
 		`)
 	})
 }
+
+func TestReplica_GetCurrentState(t *testing.T) {
+	t.Parallel()
+
+	t.Run("before initialization", func(t *testing.T) {
+		ctx := testhelper.Context(t)
+		raftCfg := raftConfigsForTest(t)
+		metrics := NewMetrics()
+		partitionID := storage.PartitionID(1)
+
+		// Create an uninitialized replica (node will be nil until Initialize is called)
+		replica, err := createRaftReplica(t, ctx, 1, "", raftCfg, partitionID, metrics)
+		require.NoError(t, err)
+		defer func() {
+			require.NoError(t, replica.Close())
+		}()
+
+		// Before initialization, node should be nil, term should be 0,
+		// index should come from logStore, and state should be follower
+		state := replica.GetCurrentState()
+		require.NotNil(t, state, "state should not be nil")
+		require.Equal(t, uint64(0), state.Term, "should return 0 term when node is nil")
+		require.GreaterOrEqual(t, state.Index, uint64(0), "should return non-negative index from logStore")
+		require.Equal(t, raft.StateFollower, state.State, "should return StateFollower when node is nil")
+	})
+
+	t.Run("after initialization", func(t *testing.T) {
+		ctx := testhelper.Context(t)
+		raftCfg := raftConfigsForTest(t)
+		metrics := NewMetrics()
+		partitionID := storage.PartitionID(1)
+
+		replica, err := createRaftReplica(t, ctx, 1, "", raftCfg, partitionID, metrics)
+		require.NoError(t, err)
+		defer func() {
+			require.NoError(t, replica.Close())
+		}()
+
+		require.NoError(t, replica.Initialize(ctx, 1))
+
+		// Wait for replica to be ready and process initial entries
+		waitTimeout := 10 * time.Second
+		require.Eventually(t, func() bool {
+			return replica.AppendedLSN() > 1
+		}, waitTimeout, 5*time.Millisecond, "replica should process initial entries")
+
+		state := replica.GetCurrentState()
+		require.NotNil(t, state, "state should not be nil")
+		require.Greater(t, state.Term, uint64(0), "should return positive term after initialization")
+		require.Greater(t, state.Index, uint64(0), "should return positive index after initialization")
+
+		// In a single-node cluster, the replica should become leader
+		require.Equal(t, raft.StateLeader, state.State, "single-node replica should become leader")
+
+		// Index should be equivalent to AppendedLSN
+		require.Equal(t, uint64(replica.AppendedLSN()), state.Index, "state.Index should equal AppendedLSN")
+
+		// Term should be consistent with what the Raft node reports
+		require.Equal(t, replica.node.Status().Term, state.Term, "state.Term should match node status")
+		require.Equal(t, replica.node.Status().RaftState, state.State, "state.State should match node status")
+	})
+
+	t.Run("consistency across multiple calls", func(t *testing.T) {
+		ctx := testhelper.Context(t)
+		raftCfg := raftConfigsForTest(t)
+		metrics := NewMetrics()
+		partitionID := storage.PartitionID(1)
+
+		replica, err := createRaftReplica(t, ctx, 1, "", raftCfg, partitionID, metrics)
+		require.NoError(t, err)
+		defer func() {
+			require.NoError(t, replica.Close())
+		}()
+
+		require.NoError(t, replica.Initialize(ctx, 1))
+
+		// Wait for replica to be ready and process initial entries
+		waitTimeout := 10 * time.Second
+		require.Eventually(t, func() bool {
+			return replica.AppendedLSN() > 1
+		}, waitTimeout, 5*time.Millisecond, "replica should process initial entries")
+
+		// Test multiple times to ensure consistency
+		for i := 0; i < 5; i++ {
+			state := replica.GetCurrentState()
+			require.NotNil(t, state, "state should not be nil (iteration %d)", i)
+
+			// Verify consistency with AppendedLSN and node status
+			require.Equal(t, uint64(replica.AppendedLSN()), state.Index,
+				"state.Index should equal AppendedLSN (iteration %d)", i)
+			require.Equal(t, replica.node.Status().Term, state.Term,
+				"state.Term should match node status (iteration %d)", i)
+			require.Equal(t, replica.node.Status().RaftState, state.State,
+				"state.State should match node status (iteration %d)", i)
+
+			// Add some delay to allow for any background processing
+			time.Sleep(10 * time.Millisecond)
+		}
+	})
+
+	t.Run("advances after log entries", func(t *testing.T) {
+		ctx := testhelper.Context(t)
+		raftCfg := raftConfigsForTest(t)
+		metrics := NewMetrics()
+		partitionID := storage.PartitionID(1)
+
+		replica, err := createRaftReplica(t, ctx, 1, "", raftCfg, partitionID, metrics)
+		require.NoError(t, err)
+		defer func() {
+			require.NoError(t, replica.Close())
+		}()
+
+		require.NoError(t, replica.Initialize(ctx, 0))
+
+		// Wait for leadership
+		waitTimeout := 10 * time.Second
+		require.Eventually(t, func() bool {
+			return replica.AppendedLSN() > 1 && replica.leadership.IsLeader()
+		}, waitTimeout, 5*time.Millisecond)
+
+		initialState := replica.GetCurrentState()
+		require.NotNil(t, initialState)
+
+		// Append a log entry
+		logEntryPath := createTestLogEntry(t, ctx, "test-content")
+		_, err = replica.AppendLogEntry(logEntryPath)
+		require.NoError(t, err)
+
+		// Index should have advanced
+		newState := replica.GetCurrentState()
+		require.NotNil(t, newState)
+		require.Greater(t, newState.Index, initialState.Index, "index should advance after appending entry")
+		require.GreaterOrEqual(t, newState.Term, initialState.Term, "term should not decrease")
+	})
+
+	t.Run("concurrent access", func(t *testing.T) {
+		ctx := testhelper.Context(t)
+		raftCfg := raftConfigsForTest(t)
+		metrics := NewMetrics()
+		partitionID := storage.PartitionID(1)
+
+		replica, err := createRaftReplica(t, ctx, 1, "", raftCfg, partitionID, metrics)
+		require.NoError(t, err)
+		defer func() {
+			require.NoError(t, replica.Close())
+		}()
+
+		require.NoError(t, replica.Initialize(ctx, 1))
+
+		// Wait for replica to be ready and process initial entries
+		waitTimeout := 10 * time.Second
+		require.Eventually(t, func() bool {
+			return replica.AppendedLSN() > 1
+		}, waitTimeout, 5*time.Millisecond)
+
+		// Test concurrent access to GetCurrentState
+		const numGoroutines = 100
+		results := make([]*ReplicaState, numGoroutines)
+		var wg sync.WaitGroup
+		wg.Add(numGoroutines)
+
+		for i := 0; i < numGoroutines; i++ {
+			go func(idx int) {
+				defer wg.Done()
+				results[idx] = replica.GetCurrentState()
+			}(i)
+		}
+
+		wg.Wait()
+
+		// All results should be consistent with current state
+		for i, result := range results {
+			require.NotNil(t, result, "result %d should not be nil", i)
+			// Results should have consistent term and index (they may vary slightly due to timing)
+			// But they should all be positive values for an active replica
+			require.Greater(t, result.Term, uint64(0), "result %d term should be positive", i)
+			require.Greater(t, result.Index, uint64(0), "result %d index should be positive", i)
+			require.NotEqual(t, raft.StateType(0), result.State, "result %d state should be valid", i)
+		}
+	})
+
+	t.Run("StateString method", func(t *testing.T) {
+		testCases := []struct {
+			state    raft.StateType
+			expected string
+		}{
+			{raft.StateFollower, "follower"},
+			{raft.StateCandidate, "candidate"},
+			{raft.StateLeader, "leader"},
+			{raft.StatePreCandidate, "precandidate"},
+		}
+
+		for _, tc := range testCases {
+			t.Run(tc.expected, func(t *testing.T) {
+				replicaState := &ReplicaState{
+					Term:  1,
+					Index: 1,
+					State: tc.state,
+				}
+				result := StateString(replicaState.State)
+				require.Equal(t, tc.expected, result)
+			})
+		}
+	})
+}
