@@ -12,24 +12,27 @@ import (
 	"syscall"
 	"testing"
 
+	"github.com/dgraph-io/badger/v4"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/git/gittest"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/config"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage/keyvalue"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage/keyvalue/databasemgr"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage/mode"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage/storagemgr/partition/snapshot"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/helper"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/log"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/testhelper"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/testhelper/testcfg"
-	"gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/git/gittest"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/config"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage/keyvalue"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage/keyvalue/databasemgr"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage/mode"
+	logmgr "gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage/storagemgr/partition/log"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage/storagemgr/partition/snapshot"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/helper"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/log"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/testhelper"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/testhelper/testcfg"
+	"gitlab.com/gitlab-org/gitaly/v18/proto/go/gitalypb"
 )
 
 type mockPartitionFactory struct {
 	new func(
+		ctx context.Context,
 		logger log.Logger,
 		partitionID storage.PartitionID,
 		db keyvalue.Transactioner,
@@ -45,6 +48,7 @@ type mockPartitionFactory struct {
 func newStubPartitionFactory() PartitionFactory {
 	return mockPartitionFactory{
 		new: func(
+			ctx context.Context,
 			logger log.Logger,
 			partitionID storage.PartitionID,
 			db keyvalue.Transactioner,
@@ -68,6 +72,7 @@ func newStubPartitionFactory() PartitionFactory {
 						return nil, ctx.Err()
 					}
 
+					kvData := make(map[string][]byte)
 					return mockTransaction{
 						commit: func(ctx context.Context) (storage.LSN, error) {
 							select {
@@ -75,11 +80,30 @@ func newStubPartitionFactory() PartitionFactory {
 								// Commits fail if partition is closed.
 								return 0, storage.ErrTransactionProcessingStopped
 							default:
+								// Commit the KV data
+								if len(kvData) > 0 {
+									err := db.Update(func(txn keyvalue.ReadWriter) error {
+										for key, value := range kvData {
+											// The db is already prefixed with the partition prefix, so just add kv/ subpath
+											partitionKey := append([]byte("kv/"), []byte(key)...)
+											if err := txn.Set(partitionKey, value); err != nil {
+												return err
+											}
+										}
+										return nil
+									})
+									if err != nil {
+										return 0, err
+									}
+								}
 								// Commits fail if context is done.
 								return 0, ctx.Err()
 							}
 						},
-						rollback: func(context.Context) error { return nil },
+						rollback:    func(context.Context) error { return nil },
+						db:          db,
+						partitionID: partitionID,
+						kvData:      kvData,
 					}, nil
 				},
 				close: func() {
@@ -88,12 +112,16 @@ func newStubPartitionFactory() PartitionFactory {
 						close(closing)
 					})
 				},
+				logReader: func() storage.LogReader {
+					return logmgr.NewManager(storageName, partitionID, stagingDir, absoluteStateDir, nil, logmgr.NewPositionTracker())
+				},
 			}
 		},
 	}
 }
 
 func (m mockPartitionFactory) New(
+	ctx context.Context,
 	logger log.Logger,
 	partitionID storage.PartitionID,
 	db keyvalue.Transactioner,
@@ -103,6 +131,7 @@ func (m mockPartitionFactory) New(
 	stagingDir string,
 ) Partition {
 	return m.new(
+		ctx,
 		logger,
 		partitionID,
 		db,
@@ -116,6 +145,7 @@ func (m mockPartitionFactory) New(
 type mockPartition struct {
 	begin       func(context.Context, storage.BeginOptions) (storage.Transaction, error)
 	run         func() error
+	logReader   func() storage.LogReader
 	close       func()
 	closeCalled atomic.Bool
 	storage.Partition
@@ -142,10 +172,17 @@ func (m *mockPartition) CloseSnapshots() error {
 	return nil
 }
 
+func (m *mockPartition) GetLogReader() storage.LogReader {
+	return m.logReader()
+}
+
 type mockTransaction struct {
 	storage.Transaction
-	commit   func(context.Context) (storage.LSN, error)
-	rollback func(context.Context) error
+	commit      func(context.Context) (storage.LSN, error)
+	rollback    func(context.Context) error
+	db          keyvalue.Transactioner
+	partitionID storage.PartitionID
+	kvData      map[string][]byte
 }
 
 func (m mockTransaction) Commit(ctx context.Context) (storage.LSN, error) {
@@ -154,6 +191,62 @@ func (m mockTransaction) Commit(ctx context.Context) (storage.LSN, error) {
 
 func (m mockTransaction) Rollback(ctx context.Context) error { return m.rollback(ctx) }
 
+func (m mockTransaction) KV() keyvalue.ReadWriter {
+	// Return a KV store that tracks changes and commits them to the real database
+	return &mockKV{
+		data:        m.kvData,
+		db:          m.db,
+		partitionID: m.partitionID,
+	}
+}
+
+type mockKV struct {
+	data        map[string][]byte
+	db          keyvalue.Transactioner
+	partitionID storage.PartitionID
+}
+
+func (m *mockKV) Get(key []byte) (keyvalue.Item, error) {
+	if value, exists := m.data[string(key)]; exists {
+		return &mockItem{value: value}, nil
+	}
+	return nil, badger.ErrKeyNotFound
+}
+
+func (m *mockKV) Set(key, value []byte) error {
+	m.data[string(key)] = value
+	return nil
+}
+
+func (m *mockKV) Delete(key []byte) error {
+	delete(m.data, string(key))
+	return nil
+}
+
+func (m *mockKV) NewIterator(opts keyvalue.IteratorOptions) keyvalue.Iterator {
+	return nil // Implement if needed
+}
+
+type mockItem struct {
+	value []byte
+}
+
+func (m *mockItem) Key() []byte {
+	return nil
+}
+
+func (m *mockItem) Value(fn func([]byte) error) error {
+	return fn(m.value)
+}
+
+func (m *mockItem) ValueCopy(dst []byte) ([]byte, error) {
+	if len(dst) < len(m.value) {
+		dst = make([]byte, len(m.value))
+	}
+	copy(dst, m.value)
+	return dst[:len(m.value)], nil
+}
+
 // blockOnPartitionClosing checks if any partitions are currently in the process of
 // closing. If some are, the function waits for the closing process to complete before
 // continuing. This is required in order to accurately validate partition state.
@@ -1596,6 +1689,89 @@ func TestStorageManager_ListPartitions(t *testing.T) {
 	})
 }
 
+func TestHasWALEntries(t *testing.T) {
+	t.Parallel()
+
+	ctx := testhelper.Context(t)
+
+	logger := testhelper.SharedLogger(t)
+	cfg := testcfg.Build(t)
+
+	dbMgr, err := databasemgr.NewDBManager(ctx, cfg.Storages, keyvalue.NewBadgerStore, helper.NewNullTickerFactory(), logger)
+	require.NoError(t, err)
+	t.Cleanup(dbMgr.Close)
+
+	storageMgr, err := NewStorageManager(
+		logger,
+		cfg.Storages[0].Name,
+		cfg.Storages[0].Path,
+		dbMgr,
+		newStubPartitionFactory(),
+		config.DefaultMaxInactivePartitions,
+		NewMetrics(cfg.Prometheus),
+	)
+	require.NoError(t, err)
+	t.Cleanup(storageMgr.Close)
+
+	for _, tc := range []struct {
+		desc           string
+		setup          func(t *testing.T, cfg config.Cfg) storage.PartitionID
+		expectedHasWAL bool
+	}{
+		{
+			desc: "no WAL directory",
+			setup: func(t *testing.T, cfg config.Cfg) storage.PartitionID {
+				partitionID := storage.PartitionID(2)
+				relativeStateDir := deriveStateDirectory(partitionID)
+				absoluteStateDir := filepath.Join(cfg.Storages[0].Path, relativeStateDir)
+				require.NoError(t, os.MkdirAll(absoluteStateDir, mode.Directory))
+
+				return partitionID
+			},
+			expectedHasWAL: false,
+		},
+		{
+			desc: "empty WAL directory",
+			setup: func(t *testing.T, cfg config.Cfg) storage.PartitionID {
+				partitionID := storage.PartitionID(3)
+				relativeStateDir := deriveStateDirectory(partitionID)
+				absoluteStateDir := filepath.Join(cfg.Storages[0].Path, relativeStateDir)
+				walDir := filepath.Join(absoluteStateDir, "wal")
+				require.NoError(t, os.MkdirAll(walDir, mode.Directory))
+
+				return partitionID
+			},
+			expectedHasWAL: false,
+		},
+		{
+			desc: "with WAL entries",
+			setup: func(t *testing.T, cfg config.Cfg) storage.PartitionID {
+				partitionID := storage.PartitionID(4)
+				relativeStateDir := deriveStateDirectory(partitionID)
+				absoluteStateDir := filepath.Join(cfg.Storages[0].Path, relativeStateDir)
+				walDir := filepath.Join(absoluteStateDir, "wal")
+				require.NoError(t, os.MkdirAll(walDir, mode.Directory))
+
+				// Create a sample WAL entry directory
+				walEntryDir := filepath.Join(walDir, "1")
+				require.NoError(t, os.MkdirAll(walEntryDir, mode.Directory))
+
+				return partitionID
+			},
+			expectedHasWAL: true,
+		},
+	} {
+		t.Run(tc.desc, func(t *testing.T) {
+			t.Parallel()
+
+			partitionID := tc.setup(t, cfg)
+			hasWAL, err := storageMgr.HasPendingWAL(ctx, partitionID)
+			require.NoError(t, err)
+			require.Equal(t, tc.expectedHasWAL, hasWAL)
+		})
+	}
+}
+
 type SyncerFunc func(ctx context.Context, rootPath, relativePath string) error
 
 func (fn SyncerFunc) SyncHierarchy(ctx context.Context, rootPath, relativePath string) error {
