@@ -8,18 +8,18 @@ import (
 	"testing"
 
 	"github.com/stretchr/testify/require"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/featureflag"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/git/catfile"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/git/gittest"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/git/housekeeping"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/git/housekeeping/config"
-	housekeepingmgr "gitlab.com/gitlab-org/gitaly/v16/internal/git/housekeeping/manager"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/git/localrepo"
-	gitalycfg "gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/config"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/grpc/protoregistry"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/testhelper"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/testhelper/testcfg"
-	"gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/featureflag"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/git/catfile"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/git/gittest"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/git/housekeeping"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/git/housekeeping/config"
+	housekeepingmgr "gitlab.com/gitlab-org/gitaly/v18/internal/git/housekeeping/manager"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/git/localrepo"
+	gitalycfg "gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/config"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/grpc/protoregistry"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/testhelper"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/testhelper/testcfg"
+	"gitlab.com/gitlab-org/gitaly/v18/proto/go/gitalypb"
 	"google.golang.org/grpc"
 	"google.golang.org/grpc/credentials/insecure"
 	healthpb "google.golang.org/grpc/health/grpc_health_v1"
@@ -508,4 +508,72 @@ func testInterceptors(t *testing.T, ctx context.Context) {
 			housekeepingManager.getOptimizeRepositoryInvocations(regularRepo.GetRelativePath()),
 			"Second regular mutator should trigger housekeeping after reaching interval")
 	})
+
+	t.Run("when snapshot stats indicate higher directory entry count than the threshold", func(t *testing.T) {
+		repo, _ := gittest.CreateRepository(t, ctx, cfg, gittest.CreateRepositoryConfig{
+			SkipCreationViaService: true,
+		})
+
+		// Setting low threshold to easily pass it
+		housekeepingMiddleware.statThreshold = 1
+
+		initialCount := housekeepingManager.getOptimizeRepositoryInvocations(repo.GetRelativePath())
+
+		_, err = gitalypb.NewRepositoryServiceClient(conn).RepositoryExists(ctx, &gitalypb.RepositoryExistsRequest{
+			Repository: repo,
+		})
+		require.NoError(t, err)
+
+		// Wait for any async housekeeping to complete
+		housekeepingMiddleware.WaitForWorkers()
+
+		newCount := housekeepingManager.getOptimizeRepositoryInvocations(repo.GetRelativePath())
+
+		// Verify that housekeeping was triggered
+		require.Equal(t,
+			testhelper.EnabledOrDisabledFlag(ctx, featureflag.HousekeepingMiddleware, initialCount+1, initialCount),
+			newCount,
+			"snapshot stats should force immediate housekeeping",
+		)
+
+		// Next request should not trigger housekeeping as it is added to the stats cache
+		_, err = gitalypb.NewRepositoryServiceClient(conn).RepositoryExists(ctx, &gitalypb.RepositoryExistsRequest{
+			Repository: repo,
+		})
+		require.NoError(t, err)
+		// Wait for any async housekeeping to complete
+		housekeepingMiddleware.WaitForWorkers()
+		// Verify that housekeeping was not triggered
+		require.Equal(t,
+			newCount,
+			housekeepingManager.getOptimizeRepositoryInvocations(repo.GetRelativePath()),
+			"snapshot stats should force immediate housekeeping",
+		)
+	})
+
+	t.Run("when snapshot stats indicate lower directory entry count than the threshold", func(t *testing.T) {
+		repo, _ := gittest.CreateRepository(t, ctx, cfg, gittest.CreateRepositoryConfig{
+			SkipCreationViaService: true,
+		})
+
+		// Setting high threshold to stay below it
+		housekeepingMiddleware.statThreshold = 1000
+
+		initialCount := housekeepingManager.getOptimizeRepositoryInvocations(repo.GetRelativePath())
+
+		_, err = gitalypb.NewRepositoryServiceClient(conn).RepositoryExists(ctx, &gitalypb.RepositoryExistsRequest{
+			Repository: repo,
+		})
+		require.NoError(t, err)
+
+		// Wait for any async housekeeping to complete
+		housekeepingMiddleware.WaitForWorkers()
+
+		// Verify that housekeeping was not triggered.
+		require.Equal(t,
+			initialCount,
+			housekeepingManager.getOptimizeRepositoryInvocations(repo.GetRelativePath()),
+			"snapshot stats should not force immediate housekeeping",
+		)
+	})
 }
